@@ -3,10 +3,11 @@
 #' @param sigMatrix  The deconvolution matrix, e.g. LM22 or MGSM27
 #' @param geneExpr  The source gene expression matrix used to calculate sigMatrix
 #' @param toPred  The gene expression to ultimately deconvolve
-#' @param heirarchData  The results of heirarchicalSplit OR heirarchicalSplit.sc (DEFAULT: NULL, ie heirarchicalSplit)
+#' @param hierarchData  The results of hierarchicalSplit OR hierarchicalSplit.sc (DEFAULT: NULL, ie hierarchicalSplit)
 #' @param pdfDir  A fold to write the pdf file to (DEFAULT: tempdir())
 #' @param oneCore Set to TRUE to disable parallelization (DEFAULT: FALSE)
 #' @param nPasses  The maximum number of iterations for spillToConvergence (DEFAULT: 100)
+#' @param remZinf Set to TRUE to remove any ratio with zero or infinity when generating gList (DEFAULT: FALSE)
 #' @export
 #' @return a matrix of cell counts
 #' @examples
@@ -15,11 +16,11 @@
 #' fullLM22 <- ADAPTS::LM22[1:30, 1:4]
 #' smallLM22 <- fullLM22[1:25,] 
 #' 
-#' cellCounts <- heirarchicalClassify(sigMatrix=smallLM22, geneExpr=fullLM22, toPred=fullLM22, 
+#' cellCounts <- hierarchicalClassify(sigMatrix=smallLM22, geneExpr=fullLM22, toPred=fullLM22, 
 #'     oneCore=TRUE, nPasses=10)
-heirarchicalClassify <- function(sigMatrix, geneExpr, toPred, heirarchData=NULL, pdfDir=tempdir(), oneCore=FALSE, nPasses=100) {
-  if(is.null(heirarchData)) {
-    heirarchData <- heirarchicalSplit(sigMatrix, geneExpr, oneCore=oneCore, nPasses=nPasses)
+hierarchicalClassify <- function(sigMatrix, geneExpr, toPred, hierarchData=NULL, pdfDir=tempdir(), oneCore=FALSE, nPasses=100, remZinf=TRUE) {
+  if(is.null(hierarchData)) {
+    hierarchData <- hierarchicalSplit(sigMatrix, geneExpr, oneCore=oneCore, nPasses=nPasses, remZinf=remZinf)
   }
 
   #Step 1: Baseline deconvolution
@@ -30,8 +31,8 @@ heirarchicalClassify <- function(sigMatrix, geneExpr, toPred, heirarchData=NULL,
 
   #Step 2: Build the clustered Deconvolution
   clusters <- NULL
-  for (i in 1:length(heirarchData$allClusters)) {
-    clusters <- rbind(data.frame(cell=heirarchData$allClusters[[i]], clust=i), clusters)
+  for (i in 1:length(hierarchData$allClusters)) {
+    clusters <- rbind(data.frame(cell=hierarchData$allClusters[[i]], clust=i), clusters)
   }
   clustIDs <- clusters$clust
   names(clustIDs) <- clusters$cell
@@ -41,10 +42,10 @@ heirarchicalClassify <- function(sigMatrix, geneExpr, toPred, heirarchData=NULL,
   #Step 3: Split the clusters based on the smaller groups
   curDecon.break.list <- list()
   for (i in as.numeric(rownames(initDecon.clust))) {
-    curCellTypes <- heirarchData$allClusters[[i]]
+    curCellTypes <- hierarchData$allClusters[[i]]
     if(length(curCellTypes) > 1) {
-      #curSigMat <- heirarchData$sigMatList[[i]][,curCellTypes,drop=FALSE]
-      curSigMat <- heirarchData$sigMatList[[i]][,,drop=FALSE]
+      #curSigMat <- hierarchData$sigMatList[[i]][,curCellTypes,drop=FALSE]
+      curSigMat <- hierarchData$sigMatList[[i]][,,drop=FALSE]
       toPred.sub <- toPred[rownames(toPred) %in% rownames(curSigMat),,drop=FALSE]
       colnames(toPred.sub) <- make.names(colnames(toPred.sub), unique=TRUE)
       toPred.sub.imp <- missForest.par(toPred.sub)
@@ -67,7 +68,7 @@ heirarchicalClassify <- function(sigMatrix, geneExpr, toPred, heirarchData=NULL,
   curDecon.new <- do.call(rbind, curDecon.break.list)
   curDecon.new <- rbind(curDecon.new, initDecon['others',,drop=FALSE])
   curDecon.new.rescale <- apply(curDecon.new, 2, function(x){100*x/sum(x)})
-  outFile <- paste('heirarchicalSplit', Sys.Date(),'pdf', sep='.')
+  outFile <- paste('hierarchicalSplit', Sys.Date(),'pdf', sep='.')
   outFile <- file.path(pdfDir, outFile)
   grDevices::pdf(outFile)
   res <- try(pheatmap::pheatmap(t(curDecon.new.rescale), fontsize=6, fontsize_row = 4, cluster_cols=FALSE, cluster_rows = TRUE), silent=TRUE)
@@ -87,6 +88,8 @@ heirarchicalClassify <- function(sigMatrix, geneExpr, toPred, heirarchData=NULL,
 #' @param geneExpr  The source gene expression matrix used to calculate sigMatrix
 #' @param oneCore Set to TRUE to disable parallelization (DEFAULT: FALSE)
 #' @param nPasses  The maximum number of iterations for spillToConvergence (DEFAULT: 100)
+#' @param deconMatrices  Optional pre-computed results from spillToConvergence (DEFAULT: NULL)
+#' @param remZinf Set to TRUE to remove any ratio with zero or infinity when generating gList (DEFAULT: FALSE)
 #' @export
 #' @return A list of clusters and a list of signature matrices for breaking those clusters
 #' @examples
@@ -95,12 +98,19 @@ heirarchicalClassify <- function(sigMatrix, geneExpr, toPred, heirarchData=NULL,
 #' fullLM22 <- ADAPTS::LM22[1:30, 1:4]
 #' smallLM22 <- fullLM22[1:25,] 
 #' 
-#' clusters <- heirarchicalSplit(sigMatrix=smallLM22, geneExpr=fullLM22, oneCore=TRUE, nPasses=10)
-heirarchicalSplit <- function(sigMatrix, geneExpr, oneCore=FALSE, nPasses=100) {
-  allClusters <- clustWspillOver(sigMatrix, geneExpr, nPasses=nPasses)
-
+#' clusters <- hierarchicalSplit(sigMatrix=smallLM22, geneExpr=fullLM22, oneCore=TRUE, nPasses=10)
+hierarchicalSplit <- function(sigMatrix, geneExpr, oneCore=FALSE, nPasses=100, deconMatrices=NULL, remZinf=TRUE) {
+  allClusters.rv <- clustWspillOver(sigMatrix, geneExpr, nPasses=nPasses, deconMatrices=deconMatrices)
+  allClusters <- allClusters.rv$allClusters
+  deconMatrices <- allClusters.rv$deconMatrices
+  
   #Step 1: Do the level 1 deconvolution
   #firstLevelDecon <- estCellPercent.DCQ(refExp=sigMatrix, geneExpr = geneExpr)
+  cNames <- sub('\\.+[0-9]+$', '', colnames(geneExpr))
+  if(!all(cNames == colnames(geneExpr))) {
+    message('Stripping .[0-9]+ from the end of gene expression column names.')
+    colnames(geneExpr) <- cNames
+  }
 
   #Make new signature matrices for each split.  How to determine # of genes for only 2 cell types?
   sigMatList <- list()
@@ -120,15 +130,13 @@ heirarchicalSplit <- function(sigMatrix, geneExpr, oneCore=FALSE, nPasses=100) {
     }
 
     colnames(curGeneExpr) <- sub('\\.[0-9]+$', '', colnames(curGeneExpr))
-    gList <- rankByT(geneExpr = curGeneExpr, qCut=0.3, oneCore=oneCore)
+    gList <- rankByT(geneExpr = curGeneExpr, qCut=0.3, oneCore=oneCore, remZinf=remZinf)
     if(length(gList) == 1) {
       otherCellType <- allClusters[[i]][!allClusters[[i]] %in% names(gList)]
       gList[[otherCellType]] <- gList[[1]]
     }
     origMatrix <- sigMatrix[,allClusters[[i]]]
-    #Top 100 by variance?
     origMatrix.sm <- origMatrix[names(utils::tail(sort(apply(origMatrix,1,stats::var)),ceiling(nrow(sigMatrix)/10))),]
-    #I will need to make AugmentSigMatrix more robust
     newMatData <- try(AugmentSigMatrix(origMatrix=origMatrix.sm, fullData=curGeneExpr, newData=curGeneExpr, gList=gList,
                                    nGenes=1:100, plotToPDF=TRUE, imputeMissing=TRUE, condTol=1.01, postNorm=FALSE,
                                    minSumToRem=NA, addTitle=paste(allClusters[[i]],collapse='_'), autoDetectMin=TRUE,
@@ -142,7 +150,7 @@ heirarchicalSplit <- function(sigMatrix, geneExpr, oneCore=FALSE, nPasses=100) {
     }
   } #for (i in 1:length(allClusters)) {
 
-  return(list(allClusters=allClusters, sigMatList=sigMatList))
+  return(list(allClusters=allClusters, sigMatList=sigMatList, deconMatrices=deconMatrices))
 }
 
 #' Build clusters based on n-pass spillover matrix
@@ -150,6 +158,7 @@ heirarchicalSplit <- function(sigMatrix, geneExpr, oneCore=FALSE, nPasses=100) {
 #' @param sigMatrix  The deconvolution matrix, e.g. LM22 or MGSM27
 #' @param geneExpr  The source gene expression matrix used to calculate sigMatrix.
 #' @param nPasses  The maximum number of iterations for spillToConvergence (DEFAULT: 100)
+#' @param deconMatrices  Optional pre-computed results from spillToConvergence (DEFAULT: NULL)
 #' @export
 #' @return  Cell types grouped by cluster
 #' @examples
@@ -158,21 +167,24 @@ heirarchicalSplit <- function(sigMatrix, geneExpr, oneCore=FALSE, nPasses=100) {
 #' fullLM22 <- ADAPTS::LM22[1:30, 1:4]
 #' smallLM22 <- fullLM22[1:25,] 
 #' 
-#' clusters <- clustWspillOver(sigMatrix=smallLM22, geneExpr=fullLM22, nPasses=10)
-clustWspillOver <- function(sigMatrix, geneExpr, nPasses=100) {
-  curGeneExpr <- geneExpr
-  naBool <- apply(curGeneExpr, 1, function(x){ any(is.na(x)) })
-  if(any(naBool))  {
-    message(paste('clustWspillOver: Removing', sum(naBool), 'genes due to NAs'))
-    curGeneExpr <- curGeneExpr[!naBool,,drop=FALSE]
-
-    keepBool <- rownames(sigMatrix) %in% rownames(geneExpr)
-    message(paste('clustWspillOver: Trimming', sum(!keepBool), '/', length(keepBool), 'genes from sigMatrix due to missingness'))
-    sigMatrix <- sigMatrix[keepBool,,drop=FALSE]
-    #Note:  Why is it imputing later if I've removed all of these??? I need to fix the NA problem better.
+#' clusters <- clustWspillOver(sigMatrix=smallLM22, geneExpr=fullLM22, nPasses=10, deconMatrices=NULL)
+clustWspillOver <- function(sigMatrix, geneExpr, nPasses=100, deconMatrices=NULL) {
+  
+  if(is.null(deconMatrices)) {
+    curGeneExpr <- geneExpr
+    naBool <- apply(curGeneExpr, 1, function(x){ any(is.na(x)) })
+    if(any(naBool))  {
+      message(paste('clustWspillOver: Removing', sum(naBool), 'genes due to NAs'))
+      curGeneExpr <- curGeneExpr[!naBool,,drop=FALSE]
+  
+      keepBool <- rownames(sigMatrix) %in% rownames(geneExpr)
+      message(paste('clustWspillOver: Trimming', sum(!keepBool), '/', length(keepBool), 'genes from sigMatrix due to missingness'))
+      sigMatrix <- sigMatrix[keepBool,,drop=FALSE]
+      #Note:  Why is it imputing later if I've removed all of these??? I need to fix the NA problem better.
+    } #if(is.null(deconMatrices)) {
+  
+    deconMatrices <- spillToConvergence(sigMatrix=sigMatrix, geneExpr=curGeneExpr, nPasses=nPasses)
   }
-
-  deconMatrices <- spillToConvergence(sigMatrix=sigMatrix, geneExpr=curGeneExpr, nPasses=nPasses)
   curExpr <- estCellCounts.nPass(geneExpr=sigMatrix, deconMatrices=deconMatrices)
 
   #Any two identical columns belong in a cluster.
@@ -186,7 +198,7 @@ clustWspillOver <- function(sigMatrix, geneExpr, nPasses=100) {
     curCor <- curCor[!clustIdx,,drop=FALSE]
   } #while(ncol(curCor) > 1) {
 
-  return(allClusters)
+  return(list(allClusters=allClusters, deconMatrices=deconMatrices))
 }
 
 #' Deconvolve with an n-pass spillover matrix
@@ -237,13 +249,20 @@ spillToConvergence <- function(sigMatrix, geneExpr, nPasses=100, plotIt=FALSE, i
     message(paste('Removing', sum(!keepBool), '/', length(keepBool), 'from geneExpr that are not in sigMatrix'))
     geneExpr <- geneExpr[,keepBool,drop=FALSE]
   }
-
+  
+  keepBool <- sub('\\.[0-9]+$', '', colnames(sigMatrix)) %in% colnames(geneExpr)
+  if (!all(keepBool)) {
+    message(paste('Removing', sum(!keepBool), '/', length(keepBool), 'from sigMatrix that are not in geneExpr'))
+    sigMatrix <- sigMatrix[,keepBool,drop=FALSE]
+  }
+  
+  
   missingGeneBool <- !rownames(sigMatrix) %in% rownames(geneExpr)
   if (sum(missingGeneBool) > 0) {
     message(paste('Removing', sum(missingGeneBool), 'that are in sigMatrix but not geneExpr'))
     sigMatrix <- sigMatrix[!missingGeneBool,]
   }
-
+  
   geneExpr.sub <- geneExpr[rownames(sigMatrix),]
   naGeneBool <- apply(geneExpr.sub, 1, function(x) {any(is.na(x))})
   if(any(naGeneBool)) {
@@ -260,7 +279,7 @@ spillToConvergence <- function(sigMatrix, geneExpr, nPasses=100, plotIt=FALSE, i
           newMatrix <- NULL
         }
       } #if(file.exists(saveFile)) {
-
+      
       if(is.null(newMatrix)) {
         message('Imputing')
         newMatrix <- missForest.par(dataMat = geneExpr.sub, parallelize = "variables")
@@ -274,7 +293,7 @@ spillToConvergence <- function(sigMatrix, geneExpr, nPasses=100, plotIt=FALSE, i
       sigMatrix <- sigMatrix[keepGenes,]
     } #if (imputNAs==TRUE) {
   }
-
+  
   cellEst <- estCellPercent.DCQ(refExpr=sigMatrix,  geneExpr=geneExpr.sub)
   if(is.null(cellEst)) {message('spillToConvergence deconvolution failed'); return(NULL)}
   newSig <- t(apply(cellEst, 1, function(x) { tapply(x, sub('\\.[0-9]+$', '', colnames(cellEst)), mean, na.rm=TRUE)}))
@@ -282,7 +301,7 @@ spillToConvergence <- function(sigMatrix, geneExpr, nPasses=100, plotIt=FALSE, i
   if(plotIt==TRUE) {pheatmap::pheatmap(t(cellEst), main='First Decon Results\n y=Purified, x=Decon As', fontsize = 4)}#, cluster_rows = FALSE, cluster_cols = FALSE)
   cellEst.last <- cellEst
   cellEst.last2 <- cellEst
-
+  
   addPassList <- list()
   addPassList[[1]] <- sigMatrix
   addPassList[[2]] <- newSig
@@ -291,35 +310,36 @@ spillToConvergence <- function(sigMatrix, geneExpr, nPasses=100, plotIt=FALSE, i
     newSig.next <- t(apply(cellEst.next, 1, function(x) { tapply(x, sub('\\.[0-9]+$', '', colnames(cellEst.next)), mean, na.rm=TRUE)}))
     rnames <- c(sort(rownames(cellEst.next)[rownames(cellEst.next) %in% colnames(cellEst.next)]), sort(rownames(cellEst.next)[!rownames(cellEst.next) %in% colnames(cellEst.next)]))
     cnames <- c(sort(colnames(cellEst.next)[colnames(cellEst.next) %in% rownames(cellEst.next)]), sort(colnames(cellEst.next)[!colnames(cellEst.next) %in% rownames(cellEst.next)]))
-    cellEst.next <- cellEst.next[rnames,cnames]
-
+    cellEst.next <- cellEst.next[rnames,colnames(cellEst.next) %in% cnames]
+    
     pairedTypes <- sort(rownames(cellEst.next)[rownames(cellEst.next) %in% colnames(cellEst.next)])
     estWself3 <- sapply(pairedTypes, function(i) {cellEst.next[i,i]})
     if(plotIt==TRUE) {graphics::barplot(estWself3, col=grDevices::rainbow(length(estWself3)), main=paste('Self Identification in Purified Samples\nPass',curPass), ylim=c(0,100), cex.names=0.66, las=2)}
-
+    
     titleStr3 <- paste0(curPass, 'x-Re-Deconvolving results with spillover matrix\nMean Self % = ', round(mean(estWself3)))
     if(plotIt==TRUE) {pheatmap::pheatmap(t(newSig.next),cluster_rows = FALSE, cluster_cols = FALSE, main=titleStr3, xlab='DCQ', ylab='Reference', fontsize = 4)} #xlab and ylab don't work
-
+    
     #addPassList[[curPass]] <- list(res=res3, cellEst=cellEst.third, estWself=estWself3, titleStr=titleStr3)
     addPassList[[curPass]] <- newSig.next
-
+    
     #Test to see if there was any change from the last pass.  If not, then stop
     diffs <- sapply(colnames(cellEst.next), function(x){ sqrt(mean((cellEst.last[,x]-cellEst.next[,x])^2))})
     if(all(diffs == 0)) { break;}
-
+    
     #Test to see if the signature matrices are oscilating
     diffs <- sapply(colnames(cellEst.next), function(x){ sqrt(mean((cellEst.last2[,x]-cellEst.next[,x])^2))})
     if(all(diffs == 0)) { break;}
-
+    
     cellEst.last2 <- cellEst.last
     cellEst.last <- cellEst.next
   } #for (curPass in 3:nPasses) {
-
+  
   if(plotIt==TRUE) {pheatmap::pheatmap(t(newSig.next),cluster_rows = TRUE, cluster_cols = TRUE, main=titleStr3, xlab='DCQ', ylab='Reference', fontsize = 4)} #xlab and ylab don't work
-
+  
   return(addPassList)
 }
 
+
 #' Build a spillover matrix, i.e. what do purified samples deconvolve as?
 #'
 #' spillExpr <- buildSpilloverMat(refExpr, geneExpr, method='DCQ')
@@ -601,6 +621,50 @@ estCellPercent.proportionsInAdmixture <- function(refExpr,  geneExpr, marker_set
   return (cellCountsPercent)
 }
 
+#' Use non-negative least squares regression to deconvolve a sample
+#'     This is going to be to simple to be useful
+#'     This might be more interesting if I used non-positive least squares to detect 'other'
+#'
+#' @param refExpr  a data frame representing immune cell expression profiles. Each row represents an expression of a gene, and each column represents a different immune cell type. colnames contains the name of each immune cell type and the rownames includes the genes' symbol. The names of each immune cell type and the symbol of each gene should be unique. Any gene with missing expression values must be excluded.
+#' @param geneExpr  a data frame representing RNA-seq or microarray gene-expression profiles of a given complex tissue. Each row represents an expression of a gene, and each column represents a different experimental sample. colnames contain the name of each sample and rownames includes the genes' symbol. The name of each individual sample and the symbol of each gene should be unique. Any gene with missing expression values should be excluded.
+#' @export
+#' @return A matrix with cell type estimates for each samples
+#' @examples
+#' #This toy example 
+#' library(ADAPTS)
+#' fullLM22 <- ADAPTS::LM22[1:30, 1:4]
+#' smallLM22 <- fullLM22[1:25,] 
+#' 
+#' cellEst <- estCellPercent.nnls(refExpr=smallLM22, geneExpr=fullLM22)
+estCellPercent.nnls <- function(refExpr,  geneExpr) {
+  marker_set <- data.frame(marker_set=rownames(refExpr))
+  if(ncol(geneExpr)==1) {geneExpr <- cbind(geneExpr, geneExpr)}
+  
+  refExprMatrix <- as.matrix(refExpr)
+  
+  refGenes <- rownames(refExprMatrix)[rownames(refExprMatrix) %in% rownames(geneExpr)]
+  refExprMatrix <- refExprMatrix[refGenes,]
+  geneExpr <- geneExpr[refGenes,]
+  
+  proportions <- matrix(nrow=ncol(refExprMatrix), ncol=ncol(geneExpr))
+  for (column in 1:ncol(geneExpr)) {
+    # SVMDECON returns the estimated proportions of each cell-type in the given sample
+    dataCol <- as.matrix(geneExpr[,column])
+    reg <- nnls::nnls(refExprMatrix, dataCol)
+    curDec <- reg$x
+    names(curDec) <- colnames(refExprMatrix)
+    proportions[, column] <- curDec
+  }
+  
+  others <- numeric(ncol(proportions))
+  proportions <- rbind(proportions, others)
+  
+  colnames(proportions) <- colnames(geneExpr)
+  rownames(proportions) <- c(colnames(refExprMatrix), "others")
+  
+  cellCountsPercent <- round(proportions * 100, 2)
+  return(cellCountsPercent)
+}
 
 #' Use weightNorm to normalize the SVM weights.  Used for SVMDECONV
 #'
